#--*-coding:utf-8-*--

class Skynauts < DiceBot
  
  def initialize
    super
    @fractionType = "omit";     #端数の処理 ("omit"=切り捨て, "roundUp"=切り上げ, "roundOff"=四捨五入)
  end
  
  
  def prefixs
    #ダイスボットで使用するコマンドを配列で列挙すること。
    ['D.*', '2[Dd]6<=.*', 'SN.*', 'NV.*', 'AVO.*', 'BOM.*']
  end
  
  def gameName
    '歯車の塔の探空士'
  end
  
  def gameType
    "Skynauts"
  end
  
  def getHelpMessage
    return <<MESSAGETEXT
◆判定　(SNn)、(2D6<=n)　n:目標値（省略時:7）
　※2d6と<=の間にスペースをいれないで下さい
◆航行チェック　(NV+n)　n:修正値（省略時:0）
◆ダメージチェック　(Dx/y@m)　x:ダメージ左側の値、y:ダメージ右側の値
　m:《弾道学》（省略可）上:8、下:2、左:4、右:6
　飛空艇シート外の座標は()が付きます。
　例） D/4　D19/2　D/3@8　D[大揺れ]/2
◆砲撃判定+ダメージチェック　(BOMn/Dx/y@m)　n:目標値（省略時:7）
　x:ダメージ左側の値、y:ダメージ右側の値
　m:《弾道学》（省略可）上:8、下:2、左:4、右:6
　例） BOM/D/4　BOM9/D19@4
◆《回避運動》　(AVOn@mXX)　n:目標値（省略時:7）
　m:回避方向。上:8、下:2、左:4、右:6、XX：ダメージチェック結果
　例）※空白は入れないでください
　AVO9@8[縦1,横4],[縦2,横6],[縦3,横8]　AVO@2[縦6,横4],[縦2,横6]
MESSAGETEXT
  end
  
  def changeText(string)
    string
  end
  
  def rollDiceCommand(command)
    debug("\n=======================================\n")
    debug("rollDiceCommand command", command)
    
    # 通常判定
    result = judgeResult(command)
    return result unless result.nil?
    
    # 航行チェック
    result = navigationResult(command)
    return result unless result.nil?
    
    # ダメージチェック
    result = fireResult(command)
    return result unless result.nil?
    
    # 砲撃判定+ダメージチェック
    result = bomberResult(command)
    return result unless result.nil?

    # 回避運動(操舵判定含む)
    result = avoidResult(command)
    return result unless result.nil? 
    
    debug("rollCommand result") 
    return nil
  end
  
  def judgeResult(command)
    return nil unless /^2[Dd]6<=(\d)$/ === command or /^SN(\d*)$/ === command
    debug("====judgeResult====")
    
    target = $1.empty? ? 7 : $1.to_i  # 目標値。省略時はエニーセブン(7)
    debug("目標値",target)
    total,diceText, = roll(2,6)
    if total <= 2 then
      result = "ファンブル！"
    elsif total <= target then
      result = "成功！"
    else 
      result = "失敗..."
    end
    text = "(2D6<=#{target}) ＞ #{total}[#{diceText}] ＞ #{total} ＞ #{result}"
    return text
  end
  
  def navigationResult(command)
    return nil unless /^NV(\+(\d+))?$/ === command
    debug("====navigationResult====")
    
    bonus = $2.to_i  # 〈操舵室〉の修正。GMの任意修正にも対応できるように(マイナスは無視)
    debug("移動修正", bonus)
    total, = roll(1,6)
    movePoint = total / 2 <= 0 ? 1 : total / 2
    movePoint = movePoint + bonus
    debug("移動エリア数", movePoint)    
    
    text = "航行チェック(最低1)　(1D6/2+#{bonus}) ＞ #{total} /2+#{bonus} ＞ "
    text += "#{total / 2 <= 0 ? 1 : total / 2}+#{bonus} ＞ #{movePoint}エリア進む"
    return text
  end
  
  def fireResult(command)
    return nil unless /^D([1-4, 6-9]*)(\[.+\])*\/(\d+)(@([2,4,6,8]))?$/ === command
    debug("====fireResult====")
    
    fireCount = $3.to_i # 砲撃回数
    fireRange = $1.to_s # 砲撃範囲
    ballistics = $5.to_i # 《弾道学》
    debug("fireCount", fireCount)
    debug("fireRange", fireRange)
    debug("ballistics", ballistics)
    if( fireCount > 25 ) then
      fireCount = 25
    end

    firePoint = firePoint(fireRange, fireCount) # 着弾座標取得（3次元配列）
    fireText = fireText(firePoint, fireCount)  # 表示用文字列作成
    if ballistics != 0 then # 《弾道学》有
      fireText += " ＞ 《弾道学》:"
      fireText += 
        case ballistics
        when 2 then "下"
        when 4 then "左"
        when 6 then "右"
        when 8 then "上"
        else
        end
      fireText += "\n ＞ "
      fireText += fireText(firePoint, fireCount, ballistics)
      
    end
    text = "#{command} ＞ #{fireText}"

    return text
  end
  
  def firePoint(fireRange, fireCount)
    debug("====firePoint====")
    firePoint = Array.new(fireCount).map{Array.new(fireRange.size + 1){Array.new(2, 0)}}
    debug("初期配列", firePoint)
    fireCount.times do |count|
      debug("\n砲撃回数", count+1)
      yPos, = roll(1, 6)  # 縦
      xPos, = roll(2, 6)  # 横
      range = 0
      
      firePoint[count][range] = [yPos, xPos]
      debug("着弾点", firePoint)
      fireRange.split(//).each { |rangeText|
        range += 1
        debug("範囲", rangeText)
        firePoint[count][range] = 
          case rangeText
          when /1/ then [yPos+1, xPos-1]
          when /2/ then [yPos+1, xPos]
          when /3/ then [yPos+1, xPos+1]
          when /4/ then [yPos, xPos-1]
          when /6/ then [yPos, xPos+1]
          when /7/ then [yPos-1, xPos-1]
          when /8/ then [yPos-1, xPos]
          when /9/ then [yPos-1, xPos+1]          
          else
          end
        debug("着弾点:範囲", firePoint)
      }
    end
    debug("\n最終着弾点", firePoint)
    
    return firePoint
  end
  
  def fireText(firePoint, fireCount, direction = 0)
    debug("====fireText====")
    fireText = ""
    firePoint.each_with_index do |point, index| 
      point.each do |y, x|
        x, y = movePoint(x, y, direction) # 《弾道学》《回避運動》などによる座標移動
        fireText += 
          if (1 <= y and y <= 6) and (2 <= x and x <= 12) then  # マップ外の座標は括弧付ける
            "[縦#{y},横#{x}]"
          else
            "([縦#{y},横#{x}])"
          end
        debug("着弾点テキスト", fireText)
      end
      fireText += index < fireCount - 1 ? "," : "" # 最後の要素以外はコンマで区切る
    end
    debug("\n最終着弾点テキスト", fireText)
    return fireText
  end
  
  def movePoint(x, y, direction)
    debug("====movePoint====")
    debug("方向", direction)
    debug("座標移動前x",x)
    debug("座標移動前y",y)
    case direction
    when 2 then y += 1  # 下
    when 4 then x -= 1  # 上
    when 6 then x += 1  # 右
    when 8 then y -= 1  # 左
    else
    end
    debug("\n座標移動後x",x)
    debug("座標移動後y",y)
    return x, y
  end
  
  def bomberResult(command)
    return nil unless /^BOM(\d*)?\/D([1-4, 6-9]*)(\[.+\])*\/(\d+)(@([2,4,6,8]))?$/ === command
    debug("====bomberResult====", command)
    direction = $6.to_i
    debug("弾道学方向", direction)
    text = "#{command} ＞ "
    text += judgeResult("SN" + $1) # 砲撃判定
    fireCommand = command.slice(/D([1-4, 6-9]*)(\[.+\])*\/(\d+)(@([2,4,6,8]))?/)  # ダメージチェック部分
    if text =~ /成功/
      text += "\n ＞ #{fireResult(fireCommand)}"
    else
      return text
    end
    return text
  end
  
  def avoidResult(command)
    return nil unless /^AVO(\d*)?(@([2,4,6,8]))(\(?\[縦\d+,横\d+\]\)?,?)+$/ === command
    debug("====avoidResult====", command)
    direction = $3.to_i
    debug("回避方向", direction)
    judgeCommand = command.slice(/^AVO(\d*)?(@([2,4,6,8]))/)  # 判定部分
    text = "#{judgeCommand} ＞ 《回避運動》"
    text += judgeResult("SN" + $1) # 操舵判定
    pointCommand = command.slice(/(\(?\[縦\d+,横\d+\]\)?,?)+/)  # 砲撃座標
    if text =~ /成功/
      firePoint, fireCount = scanFire(pointCommand)
      text += "\n ＞ #{pointCommand}"
      text += " ＞ 《回避運動》:"
      text += 
        case direction
        when 2 then "下"
        when 4 then "左"
        when 6 then "右"
        when 8 then "上"
        else
        end
      text += "\n ＞ "
      text += fireText(firePoint, fireCount, direction)

    else
      return text
    end
    return text
  end
  
  def scanFire(command)
    debug("====scanFirePoint====", command)
    command = command.gsub(/\(|\)/, "") # 正規表現が大変なので最初に括弧外しておく
    fireCount = command.scan(/\],\[/).size + 1
    fireSize = command.scan(/\[縦\d+,横\d+\]/).size / fireCount # fireRangeと違い、こっちは数値
    debug("回避用砲撃回数", fireCount)
    debug("回避用範囲大きさ", fireSize)

    firePoint = Array.new(fireCount).map{Array.new(fireSize){Array.new(2, 0)}}
    debug("初期配列", firePoint)
    
    # 力技でごめんなさい
    command.split(/\],/).each_with_index { |pointText, count| # 一組ずつに分ける(縦y,横xの単位)
      debug("pointText", pointText)
      pointText.split(/\]/).each_with_index { |point, range| # D以外の砲撃範囲がある時に必要
        debug("point", point)
        point.split(/,/).each_with_index { |nam, i|
          debug("nam", nam)
          debug("count", count)
          debug("range", range)
          debug("i", i)
          debug("代入>>", nam.gsub(/[^\d]/, "").to_i)
          firePoint[count.to_i][range.to_i][i.to_i] = nam.gsub(/[^\d]/, "").to_i
          debug("着弾点", firePoint)
        }
      }
    }
    return firePoint, fireCount
  end
  
  # ゲーム別成功度判定(2D6)
  def check_2D6(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    ''
  end
  
  # ゲーム別成功度判定(nD6)
  def check_nD6(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    ''
  end
  
  # ゲーム別成功度判定(nD10)
  def check_nD10(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    ''
  end
  
  # ゲーム別成功度判定(1d100)
  def check_1D100(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    ''
  end

  # ゲーム別成功度判定(1d20)
  def check_1D20(total_n, dice_n, signOfInequality, diff, dice_cnt, dice_max, n1, n_max)
    ''
  end
  
  
  #以下のメソッドはテーブルの参照用に便利
  #get_table_by_2d6(table)
  #get_table_by_1d6(table)
  #get_table_by_nD6(table, 1)
  #get_table_by_nD6(table, count)
  #get_table_by_1d3(table)
  #get_table_by_number(index, table)
  #get_table_by_d66(table)
  
  #getDiceList を呼び出すとロース結果のダイス目の配列が手に入ります。
end
